from django.apps import AppConfig


class OpenaiSimpleChatDjangoTemplateExampleConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'openai_simple_chat_django_template_example'
